from __future__ import annotations
import base64
import hashlib
import json
import os
import random
import string
from cryptography.fernet import Fernet
from termcolor import colored

silent = False


class Data:
    def __init__(self, key: bytes or None, storage="storage-encrypted/"):
        if key is None:
            self.crypted = False
            if storage == "storage-encrypted/":
                storage = "storage-raw/"
        else:
            self.crypted = True
        if not storage.endswith("/"):
            storage += "/"
        self.key = key
        self.storage = storage
        if self.crypted:
            self.encrypt = Fernet(key)

    @staticmethod
    def new(key: str or None = None, storage="storage-encrypted/"):
        if key is not None:
            input_bytes = key.encode('utf-8')
            hashed_bytes = hashlib.sha256(input_bytes).digest()
            fernet_key = base64.urlsafe_b64encode(hashed_bytes)

            keys_hash_list = Data(None, "saved-keys/").pull("list")
            if keys_hash_list is None:
                keys_hash_list = []
            hashed_key = hashlib.sha256(key.encode('utf-8')).hexdigest()
            if hashed_key not in keys_hash_list:
                if not silent:
                    print(f"{colored('Warning, a new key has been generated', 'red')}. The key is: {colored(fernet_key.decode(), 'cyan')} text version is: {colored(key, 'cyan')}")
                keys_hash_list.append(hashed_key)
                Data(None, "saved-keys/").push(keys_hash_list, "list")
            else:
                if not silent:
                    print(f"{colored('The has already existed', 'light_green')}. The key is: {colored(fernet_key.decode(), 'cyan')} text version is: {colored(key, 'cyan')}")
            return Data(fernet_key, storage)
        elif key == "":
            if not silent:
                print(f"{colored('Warning, a raw data storage has been created', 'red')}.")
            return Data(None, storage)
        else:
            key = "".join(random.choices(string.ascii_letters + string.digits, k=20))
            input_bytes = key.encode('utf-8')
            hashed_bytes = hashlib.sha256(input_bytes).digest()
            fernet_key = base64.urlsafe_b64encode(hashed_bytes)

            keys_hash_list = Data(None, "saved-keys/").pull("list")
            if keys_hash_list is None:
                keys_hash_list = []
            hashed_key = hashlib.sha256(key.encode('utf-8')).hexdigest()
            if hashed_key not in keys_hash_list:
                if not silent:
                    print(f"{colored('Warning, a new key has been generated', 'red')}. The key is: {colored(fernet_key.decode(), 'cyan')} text version is: {colored(key, 'cyan')}")
                keys_hash_list.append(hashed_key)
                Data(None, "saved-keys/").push(keys_hash_list, "list")
            else:
                if not silent:
                    print(f"{colored('The has already existed', 'light_green')}. The key is: {colored(fernet_key.decode(), 'cyan')} text version is: {colored(key, 'cyan')}")

            return Data(fernet_key, storage), key

    def raw_pull(self, path: str, split_mode: str = ".", extension: str = ".sddata") -> str:
        file_path = self.storage + path + extension if split_mode not in path else self.storage + "".join([path_part + "/" for path_part in path.split(split_mode)]).rstrip("/") + extension
        try:
            with open(file_path, "r") as data_file:
                try:
                    return data_file.read()
                except:
                    return
        except:
            return

    def pull(self, path: str, split_mode: str = ".", extension: str = ".sddata", encypted: bool = True):
        if not self.crypted:
            encypted = False
        if encypted:
            try:
                value = self.encrypt.decrypt(self.raw_pull(path, split_mode, extension)).decode()
                return json.loads(value)
            except:
                return
        else:
            try:
                return json.loads(self.raw_pull(path, split_mode, extension))
            except:
                return

    def raw_push(self, data, path: str, split_mode: str = ".", extension: str = ".sddata") -> bool:
        file_path = self.storage + path + extension if split_mode not in path else self.storage + "".join([path_part + "/" for path_part in path.split(split_mode)]).rstrip("/") + extension
        dirs = "".join(folder + "/" for folder in file_path.split("/")[:-1]).rstrip("/")
        os.makedirs(dirs, exist_ok=True)
        try:
            with open(file_path, "w") as data_file:
                try:
                    data_file.write(data)
                    return True
                except:
                    return False
        except:
            return False

    def push(self, data, path: str, split_mode: str = ".", extension: str = ".sddata", encypted: bool = True) -> bool:
        if not self.crypted:
            encypted = False
        if encypted:
            try:
                if self.raw_push(self.encrypt.encrypt(json.dumps(data).encode()).decode(), path, split_mode, extension):
                    return True
                else:
                    return False
            except:
                return False
        else:
            try:
                if self.raw_push(json.dumps(data), path, split_mode, extension):
                    return True
                else:
                    return False
            except:
                return False
